var ogma = new Ogma({
  container: 'graph-container',
  renderer: 'canvas'
});

// show graph size stats
function showStats() {
  document.getElementById('stats').innerHTML =
    ogma.getNodes().size + ' nodes, ' + ogma.getEdges().size + 'edges';
}
ogma.events
  .onEdgesAdded(showStats)
  .onNodesAdded(showStats)
  .onNodesRemoved(showStats);

// load graph
ogma.parse.gexfFromUrl('miserables.gexf').then((graph) => {
  ogma.setGraph(graph);
  ogma.view.locateGraph();
});

/* Set up a keyboard shortcuts to trigger the lasso, zoom in and out, and run layout */
function toggleLasso() {
  if (ogma.tools.lasso.enabled()) {
    ogma.tools.lasso.disable();
  } else {
    ogma.tools.lasso.enable();
  }
}

ogma.events
  .onKeyPress('l', toggleLasso)
  .onKeyPress('space', function() {
    var selected = ogma.getSelectedNodes();
    ogma.layouts.forceLink({
      nodes: selected.size === 0 ? ogma.getNodes() : selected,
      locate: true,
      scalingRatio: 600
    });
  })
  .onKeyPress('+', function() {
    ogma.view.zoomIn({ duration: 200 });
  })
  .onKeyPress('-', function() {
    ogma.view.zoomOut({ duration: 200 });
  })
  .onKeyPress('c', function() {
    ogma.view.locateGraph({ duration: 200 });
  });

// switch rectangle select on drag start with the 'ctrl' key pressed
ogma.events.onDragStart(function () {
  if (ogma.keyboard.isKeyPressed('ctrl')) {
    ogma.tools.rectangleSelect.enable();
  }
});


/* Set up a handler when nodes are selected - show in the toolbar */
document.getElementById('selected-nodes').addEventListener('click', function(evt) {
  if (/node/.test(evt.target.className)) {
    var id = evt.target.getAttribute('data-id');
    ogma.getNode(id).setSelected(false);
  }
});

function onSelectionChange() {
  var selected = ogma.getSelectedNodes();
  /* Display the number of selected node outside of Ogma */
  document.getElementById('selected-nodes-number').innerHTML = selected.size;
  document.getElementById('selected-nodes').innerHTML =
    selected.getAttribute('text').map(function(text, i) {
      var node = selected.get(i);
      var id = node.getId();
      var color = node.getAttribute('color');
      return '<span class="node" style="background-color: ' + color + '" ' +
                   'title="Unselect" data-id="' + id + '">' +
                text + '</span>';
    }).join('');
}
ogma.events
  .onNodesSelected(onSelectionChange)
  .onNodesUnselected(onSelectionChange)
  .onNodesRemoved(onSelectionChange);

var hoveredContainer = document.getElementById('hovered-nodes');
ogma.events.onMouseMove(function(evt) {
  var hovered = ogma.getHoveredElement();
  if (hovered && hovered.isNode) {
    var color = hovered.getAttribute('color');
    var text  = hovered.getAttribute('text');
    hoveredContainer.innerHTML =
      '<span class="node" style="background-color: ' + color + '">' +
                text + '</span>';
  } else {
    hoveredContainer.innerHTML = '';
  }
});

// Dragging behavior
// style for the dragged nodes - red background
ogma.createClass('dragged', {
  nodeAttributes: {
    color: 'red'
  }
});
// style for the rest of the nodes - grey background
ogma.createClass('static', {
  nodeAttributes: {
    color: '#888888'
  }
});

// assign classes on drag start and remove them when the interaction is finished
ogma.events
  .onNodeDragStart(function(evt) {
    // add visual style to the nodes that are not affected
    var nodes = evt.nodes;
    nodes.addClass('dragged');
    nodes.inverse().addClass('static');
  })
  .onNodeDragEnd(function(evt) {
    // reset visual style for the nodes that are not affected
    var nodes = evt.nodes;
    nodes.removeClass('dragged');
    nodes.inverse().removeClass('static');
  });


// context menu routines
var menu = document.getElementById('nodes-menu');
var hideMenuTimer = 0;

function hideMenu () {
  menu.classList.add('hidden');
  currentNode = null;
  clearTimeout(hideMenuTimer);
}

menu.addEventListener('mouseover', function () {
  clearTimeout(hideMenuTimer);
});
menu.addEventListener('mouseout', function() {
  hideMenuTimer = setTimeout(hideMenu, 1000);
});
menu.addEventListener('click', function(evt) {
  var action = evt.target.getAttribute('data-action');
  if (currentNodes) {
    if (action === 'select') {
      currentNodes.setSelected(true);
    } else if (action === 'delete') {
      ogma.removeNodes(currentNodes);
    } else if (action === 'connect') {
      ogma.tools.connectNodes.enable();
    } else if (action === 'center') {
      ogma.layouts.radial({
        centralNode: currentNodes.get(0),
        radiusDelta: 300,
        repulsion: 5,
        locate: true
      });
    }
  }
  hideMenu();
});

function showMenu(x, y, nodes) {
  menu.style.top  = y + 'px';
  menu.style.left = x + 10 + 'px';

  clearTimeout(hideMenuTimer);

  menu.classList.remove('hidden');
  if (nodes.size === 1) {
    menu.classList.remove('group');
  } else {
    menu.classList.add('group');
  }
}


/* context menu */
var currentNodes;
ogma.events.onClick(function(evt) {
  var target = evt.target;
  if (evt.button === 'right' && target && target.isNode) {
    var selected = ogma.getSelectedNodes();
    var group = false;

    // different items for menu for group or a single node
    // make sure we have a collection for consistency
    if (selected.includes(target)) {
      currentNodes = selected;
    } else {
      currentNodes = target.toList();
    }
    var position = ogma.view.graphToScreenCoordinates(target.getPosition());
    showMenu(position.x, position.y, currentNodes);
  }
});

